@@ -1206,7 +1206,6 @@ ext2fs_open(const char *name, unsigned char ftype)
     if ((fs->fd = open(name, O_RDONLY)) < 0)
 		error("%s: open %s: %m", myname, name);
 
-
 	fs->ftype = ftype;
 	fs->flags = 0;
 	fs->flags |= FS_HAVE_DTIME;
@@ -1226,8 +1225,10 @@ ext2fs_open(const char *name, unsigned char ftype)
 	 * Verify we are looking at an EXT2FS image
 	 */
 
-	if (guessu16(fs, ext2fs->fs->s_magic, EXT2FS_FS_MAGIC))
-		error("Error: %s is not an EXT2FS file system", name);
+	if (guessu16(fs, ext2fs->fs->s_magic, EXT2FS_FS_MAGIC)) {
+		error("Error: %s is not an %s file system", name,
+		  (ftype == EXT3FS_1)?"EXT3FS":"EXT2FS");
+	}
 
     if (verbose) {
     	if (getu32(fs, ext2fs->fs->s_feature_ro_compat) & 
