@@ -46,8 +46,8 @@ static int32_t	secwest;	/* Number of seconds west of UTC for time
  * Return the entry in the File Allocation Table (FAT) for the given 
  * cluster
  *
- * The appropriate entry size is then called
- *
+ * The return value is in clusters and may need to be coverted to
+ * sectors by the calling function
  */
 static u_int32_t 
 getFAT(FATFS_INFO *fatfs, u_int32_t clust)
@@ -74,7 +74,7 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 		if ((sect < fatfs->table->addr) || 
 		  (sect >= (fatfs->table->addr + FAT_CACHE_S)) ||
 		  (-1 == fatfs->table->addr) ) {
-			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "");
+			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "getFAT FAT12");
 		}
 
 		/* get the offset into the cache */
@@ -86,7 +86,7 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 		 * size must therefore be at least 2 sectors large 
 		 */
 		if (offs == (FAT_CACHE_B - 1) )  {
-			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "");
+			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "getFAT FAT12 - overlap");
 
 			offs = (sect - fatfs->table->addr)*fatfs->ssize + 
 			  (clust + clust / 2) % fatfs->ssize; 
@@ -116,7 +116,7 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 		if ((sect < fatfs->table->addr) || 
 		  (sect >= (fatfs->table->addr + FAT_CACHE_S)) ||
 		  (-1 == fatfs->table->addr) ) 
-			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "");
+			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "getFAT FAT16");
 
 		/* get pointer to entry in current buffer */
 		ptr = fatfs->table->data +
@@ -138,7 +138,7 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 		if ((sect < fatfs->table->addr) || 
 		  (sect >= (fatfs->table->addr + FAT_CACHE_S)) ||
 		  (-1 == fatfs->table->addr) ) 
-			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "");
+			fs_read_block (fs, fatfs->table, FAT_CACHE_B, sect, "getFAT FAT32");
 
 
 		/* get pointer to entry in current buffer */
@@ -156,6 +156,12 @@ getFAT(FATFS_INFO *fatfs, u_int32_t clust)
 		break;
 	}
 
+	if (verbose)
+		fprintf (logfp,
+		  "fatfs_getFAT: Lookup Cluster %lu to %lu\n", 
+		  (ULONG)clust, (ULONG)retval);
+
+
 	return retval;
 }
 
@@ -288,6 +294,11 @@ fatfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
     if (last < fs->start_block || last > fs->last_block)
         error("%s: invalid last block number: %lu", myname, (ULONG) last);
 
+    if (verbose)
+		fprintf (logfp,
+		  "fatfs_block_walk: Block Walking %lu to %lu\n", 
+		  (ULONG)start, (ULONG)last);
+
 	/* cycle through block addresses */
 	for (addr = start; addr <= last; addr++) {
 
@@ -302,7 +313,7 @@ fatfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 			myflags |= FS_FLAG_META;
 
 		if ((flags & myflags) == myflags) {
-			fs_read_block(fs, fs_buf, fs->block_size, addr, "data block");
+			fs_read_block(fs, fs_buf, fs->block_size, addr, "block_walk: data block");
 			if (WALK_STOP == action(fs, addr, fs_buf->data, myflags, ptr)) {
 				fs_buf_free(fs_buf);
 				return;
@@ -473,7 +484,7 @@ fatfs_copy_inode(FATFS_INFO *fatfs, FS_INODE *fs_inode)
 {
     fatfs_dentry    *in = fatfs->dep;
 	int 		cnum, i;
-	u_int32_t 	mask = 0, dcnt, clust;
+	u_int32_t 	dcnt, clust;
 	int32_t		size;
 	FS_INFO *fs = (FS_INFO *)&fatfs->fs_info;
 
@@ -550,17 +561,8 @@ fatfs_copy_inode(FATFS_INFO *fatfs, FS_INODE *fs_inode)
 	fs_inode->name->name[i] = '\0';
 
 
-	/* get the mask */
-	if (fatfs->fs_info.ftype == MS16_FAT)
-		mask = FATFS_16_MASK;
-	else if (fatfs->fs_info.ftype == MS32_FAT)
-		mask = FATFS_32_MASK;
-	else if (fatfs->fs_info.ftype == MS12_FAT)
-		mask = FATFS_12_MASK;
-
-
 	/* get the starting cluster */
-	fs_inode->direct_addr[0] = FATFS_DENTRY_CLUST(fs, in) & mask;
+	fs_inode->direct_addr[0] = FATFS_DENTRY_CLUST(fs, in) & fatfs->mask;
 
 	/* wipe the remaining fields */
 	for (dcnt = 1 ; dcnt < fs_inode->direct_count; dcnt++)
@@ -576,7 +578,7 @@ fatfs_copy_inode(FATFS_INFO *fatfs, FS_INODE *fs_inode)
 		/* count the total number of clusters in this file */
 		clust = FATFS_DENTRY_CLUST(fs, in);
 		cnum = 0;
-		while ((clust) && (0 == FATFS_ISEOF(clust, mask))) {
+		while ((clust) && (0 == FATFS_ISEOF(clust, fatfs->mask))) {
 			cnum++;
 			clust = getFAT(fatfs, clust);
 		}
@@ -679,7 +681,8 @@ fatfs_isdentry(FATFS_INFO *fatfs, fatfs_dentry *de)
 		if ( (getu16(fs, de->ctime) != 0) && 
 		  (FATFS_ISTIME(getu16(fs, de->ctime)) == 0) ) 
 			return 0;
-		else if (FATFS_ISTIME(getu16(fs, de->wtime)) == 0) 
+		else if ( (getu16(fs, de->wtime) != 0) && 
+		  (FATFS_ISTIME(getu16(fs, de->wtime)) == 0) ) 
 			return 0;
 		else if ( (getu16(fs, de->cdate) != 0) && 
 		  (FATFS_ISDATE(getu16(fs, de->cdate)) == 0) ) 
@@ -689,6 +692,11 @@ fatfs_isdentry(FATFS_INFO *fatfs, fatfs_dentry *de)
 			return 0;
 		else if (FATFS_ISDATE(getu16(fs, de->wdate)) == 0) 
 			return 0;
+
+		/* verify the starting cluster is small enough */
+		else if ((FATFS_DENTRY_CLUST(fs, de) > (fatfs->clustcnt + 1)) &&
+		  (FATFS_ISEOF(FATFS_DENTRY_CLUST(fs, de), fatfs->mask) == 0)) 
+			return 0;
 	}
 
 	if ((de->attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN)
@@ -728,6 +736,10 @@ fatfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
     if (last < fs->first_inum || last > fs->last_inum || last < start)
         error("%s: invalid last inode number: %lu", myname, (ULONG) last);
 
+    if (verbose)
+		fprintf (logfp,
+		  "fatfs_inode_walk: Inode Walking %lu to %lu\n", 
+		  (ULONG)start, (ULONG)last);
 
 	/* The root_inum is reserved for the root directory, which does
 	 * not have a dentry in FAT, so we make one up
@@ -807,7 +819,7 @@ fatfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 
 		/* read it */
 		fs_read_block(fs, fatfs->dinodes, fatfs->ssize, sect, 
-		  "fatfs_inode_walk");
+		  "fatfs_inode_walk: block of inodes");
 
 		/* if it is not a bunch of dentries, then skip it */
 		if (0 == fatfs_isdentry(fatfs, (fatfs_dentry *)fatfs->dinodes->data)) 
@@ -815,13 +827,18 @@ fatfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 
 		inum = FATFS_SECT_2_INODE(fatfs, sect);
 
+	    if (verbose)
+			fprintf (logfp,
+			  "fatfs_inode_walk: Processing sector %lu starting at inode %lu\n", 
+			  (ULONG)sect, (ULONG)inum);
+
 		/* cycle through the sectors, if we are in a cluster */
 		for (i = 0; i < sincr; i++) {
 
 			/* read the next sector, if needed */
 			if (i != 0) 
 				fs_read_block(fs, fatfs->dinodes, fatfs->ssize,
-				  sect + i, "fatfs_inode_walk");
+				  sect + i, "fatfs_inode_walk: sectors");
 
 			fatfs->dep = (fatfs_dentry *)fatfs->dinodes->data;
 
@@ -879,6 +896,11 @@ fatfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 
 				fatfs_copy_inode(fatfs, fs_inode);
 				fs_inode->flags = myflags;
+					
+    			if (verbose)
+					fprintf (logfp,
+					  "fatfs_inode_walk: Directory Entry %lu (%lu) at sector %lu\n",
+					  (ULONG)inum, (ULONG)didx, (ULONG)sect + i);
 
 				if (WALK_STOP == action(fs, inum, fs_inode, myflags, ptr)) {
 					fs_inode_free(fs_inode);
@@ -960,7 +982,7 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
     FATFS_INFO 	*fatfs = (FATFS_INFO *) fs;
 	int 		myflags = FS_FLAG_CONT | FS_FLAG_ALLOC, i;
 	OFF_T		size, sbase, clust;
-	u_int32_t	len, mask = 0;
+	u_int32_t	len;
 	FS_BUF		*fs_buf;
 
 	fs_buf = fs_buf_alloc(fatfs->ssize);
@@ -969,24 +991,34 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 	else
 		size = fs_inode->size;
 
-	/* get the mask */
-	if (fs->ftype == MS16_FAT)
-		mask = FATFS_16_MASK;
-	else if (fs->ftype == MS32_FAT)
-		mask = FATFS_32_MASK;
-	else if (fs->ftype == MS12_FAT)
-		mask = FATFS_12_MASK;
-
 	clust = fs_inode->direct_addr[0];
 
+	if ((clust > (fatfs->clustcnt + 1)) &&
+	  (FATFS_ISEOF(clust, fatfs->mask) == 0)) {
+		if (flags & FS_FLAG_NOABORT) {
+			fprintf(stderr, "fatfs_file_walk: Warning: Invalid starting cluster address in Directory Entry (too large): %lu",
+			  (ULONG)clust);
+			return;
+		}
+		else {
+			error("fatfs_file_walk: Invalid starting cluster address in Directory Entry (too large): %lu",
+			  (ULONG)clust);
+		}
+	}
+
 	/* this is the root directory entry, special case: it is not in the FAT */
 	if ((fs->ftype != MS32_FAT) && (clust == 1)) {
 		int snum = fatfs->firstclustsect - fatfs->firstdatasect;
+
+		if (verbose)
+			fprintf (logfp,
+			  "fatfs_file_walk: Walking Root Directory\n");
+
 		for (i = 0; i < snum; i++) {
 
 			if ((flags & FS_FLAG_AONLY) == 0)
 				fs_read_block(fs, fs_buf, fatfs->ssize, 
-				  fatfs->rootsect + i, "");
+				  fatfs->rootsect + i, "file_walk: root directory");
 
 			action (fs, fatfs->rootsect + i, fs_buf->data, fatfs->ssize, 
 			  myflags, ptr);
@@ -994,17 +1026,21 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 	}
 	else {
 
-		while ((clust & mask) > 0 && size > 0 && 
-		  (0 == FATFS_ISEOF(clust, mask)) )  {
+		while ((clust & fatfs->mask) > 0 && size > 0 && 
+		  (0 == FATFS_ISEOF(clust, fatfs->mask)) )  {
 
-			sbase = fatfs->firstclustsect + ((clust & mask) - 2) * fatfs->csize;
+			/* Convert the cluster to a sector address */
+			sbase = fatfs->firstclustsect + ((clust & fatfs->mask) - 2) * fatfs->csize;
 
 			if (sbase > fs->last_block) {
 				fs_buf_free(fs_buf);
-				if (flags & FS_FLAG_NOABORT)
+				if (flags & FS_FLAG_NOABORT) {
+					fprintf(stderr, "fatfs_file_walk: Invalid sector address in FAT (too large): %lu",
+					  (ULONG)sbase);
 					return;
+				}
 				else
-					error ("Invalid sector address FAT (too large): %lu",
+					error ("fatfs_file_walk: Invalid sector address in FAT (too large): %lu",
 					  (ULONG)sbase);
 			}
 
@@ -1015,8 +1051,14 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 				else
 					len = (size < fatfs->ssize) ? size : fatfs->ssize;
 
+				if (verbose)
+					fprintf (logfp,
+					  "fatfs_file_walk: Processing %d bytes of sector %lu\n",
+					  len, (ULONG)(sbase + 1));
+
 				if ((flags & FS_FLAG_AONLY) == 0)
-					fs_read_block(fs, fs_buf, fatfs->ssize, sbase + i, "");
+					fs_read_block(fs, fs_buf, fatfs->ssize, sbase + i, 
+					"file_walk: sectors");
 
 				action (fs, sbase + i, fs_buf->data, len, myflags, ptr);
 				size -= len;
@@ -1034,7 +1076,7 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 static void
 fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 {
-	int i, mask = 0;
+	int i;
 	u_int32_t	next, snext, sstart, send;
 	FATFS_INFO *fatfs = (FATFS_INFO *)fs;
 	fatfs_sb *sb = fatfs->sb;
@@ -1109,13 +1151,6 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "\nFAT CONTENTS (in sectors)\n");
 	fprintf(hFile, "--------------------------------------------\n");
 
-	if (fatfs->fs_info.ftype == MS16_FAT)
-		mask = FATFS_16_MASK;
-	else if (fatfs->fs_info.ftype == MS32_FAT)
-		mask = FATFS_32_MASK;
-	else if (fatfs->fs_info.ftype == MS12_FAT)
-		mask = FATFS_12_MASK;
-
 	/* 'sstart' marks the begining of the current run to print */
 	sstart = fatfs->firstclustsect;
 
@@ -1129,18 +1164,18 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 
 		/* get the next cluster */
 		next = getFAT(fatfs, i);	
-		snext = fatfs->firstclustsect + ((next & mask) - 2) * fatfs->csize;
+		snext = fatfs->firstclustsect + ((next & fatfs->mask) - 2) * fatfs->csize;
 
 		/* we are also using the next sector (clust) */
-		if ((next & mask) == (i + 1)) {
+		if ((next & fatfs->mask) == (i + 1)) {
 			continue;
 		}
 
 		/* The next clust is either further away or the clust is available,
 		 * print it if is further away 
 		 */
-		else if ((next & mask)) {
-			if (FATFS_ISEOF(next, mask)) 
+		else if ((next & fatfs->mask)) {
+			if (FATFS_ISEOF(next, fatfs->mask)) 
 				fprintf (hFile, "%d-%d (%d) -> EOF\n", sstart, send, 
 				  send - sstart + 1);
 			else
@@ -1287,10 +1322,23 @@ fatfs_open(const char *name, unsigned char ftype)
 			error ("Error: %s is not a FATFS file system", name);
 
 	fatfs->ssize = getu16(fs, fatsb->ssize);
+	if (fatfs->ssize % FATFS_DEV_BSIZE) {
+		error ("Error: sector size (%d) is not a multiple of device size (%d)\nDo you have a disk image instead of a partition image?",
+		  fatfs->ssize, FATFS_DEV_BSIZE);
+	}
+
 	fatfs->csize = fatsb->csize;				/* cluster size */
+	if (fatfs->csize == 0) 
+		error ("Error: %s is not a FATFS file system (cluster size)", name); 
+
 	fatfs->numfat = fatsb->numfat;				/* number of tables */
+	if (fatfs->numfat == 0) 
+		error ("Error: %s is not a FATFS file system (number of FATs)", name); 
+
+	/* We can't do a sanity check on this b.c. FAT32 has a value of 0 */
 	fatfs->numroot = getu16(fs, fatsb->numroot);	/* num of root entries */
 
+
 	/* if sectors16 is 0, then the number of sectors is stored in sectors32 */
 	if (0 == (fatfs->sectors = getu16(fs, fatsb->sectors16)))
 		fatfs->sectors = getu32(fs, fatsb->sectors32);
@@ -1303,7 +1351,14 @@ fatfs_open(const char *name, unsigned char ftype)
 	if (0 == (fatfs->sectperfat = getu16(fs, fatsb->sectperfat16)))
 		fatfs->sectperfat = getu32(fs, fatsb->a.f32.sectperfat32);
 
+	if (fatfs->sectperfat == 0) 
+		error ("Error: %s is not a FATFS file system (invalid sectors per FAT)",
+		  name);
+
 	fatfs->firstfatsect = getu16(fs, fatsb->reserved); 	
+	if ((fatfs->firstfatsect == 0) || (fatfs->firstfatsect > fatfs->sectors))
+		error ("Error: %s is not a FATFS file system (invalid first FAT sector %lu)", 
+		  name, (ULONG)fatfs->firstfatsect);
 
 	/* The data area starts after the FAT */
 	fatfs->firstdatasect = fatfs->firstfatsect + 
@@ -1337,7 +1392,7 @@ fatfs_open(const char *name, unsigned char ftype)
 		 * as FAT32, so we will check fields.  numroot is always 0 for 32
 		 */
 		else {
-			if (getu16(fs, fatsb->numroot) == 0)
+			if (getu16(fs, fatsb->numroot) == 0) 
 				ftype = MS32_FAT;
 			else 
 				ftype = MS16_FAT;
@@ -1354,6 +1409,16 @@ fatfs_open(const char *name, unsigned char ftype)
 			error ("Invalid FAT32 image (numroot != 0)");
 	}
 
+	/* Set the mask to use on the cluster values */
+	if (ftype == MS12_FAT)
+		fatfs->mask = FATFS_12_MASK;
+	else if (ftype == MS16_FAT)
+		fatfs->mask = FATFS_16_MASK;
+	else if (ftype == MS32_FAT)
+		fatfs->mask = FATFS_32_MASK;
+	else
+		error ("Unknown FAT type in fatfs_open: %d\n", ftype);
+
 	/* the root directories are always after the FAT for FAT12 and FAT16,
 	 * but are dynamically located for FAT32
 	 */
