@@ -617,13 +617,26 @@ ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
         read_count = (length < buf->size ? length : buf->size);
 
     if (addr > fs->last_block) {
-        if (flags & FS_FLAG_NOABORT) 
+        if (flags & FS_FLAG_NOABORT)  {
+			fprintf (stderr, "Invalid direct block address (too large): %lu",
+			  (ULONG)addr);
             return 0;
-		else
+		}
+		else {
 			error ("Invalid direct block address (too large): %lu",
 			  (ULONG)addr);
+		}
     }    
 
+	/* Check if this goes over the end of the image 
+	 * This exists when the image size is not a multiple of the block
+	 * size and read_count is for a full block.
+	 * 
+	 */
+	if (addr + (read_count/fs->block_size) > fs->last_block) {
+		read_count = (fs->last_block - addr + 1) * fs->block_size;
+	}
+
 	/* all direct pointers are content */
     myflags = FS_FLAG_CONT;
 
@@ -671,11 +684,15 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], OFF_T length,
 ;   
     
     if (addr > fs->last_block) {
-        if (flags & FS_FLAG_NOABORT) 
+        if (flags & FS_FLAG_NOABORT)  {
+        	fprintf (stderr, "Invalid indirect block address (too large): %lu",
+			  (ULONG)addr);
             return 0;
-		else 
+		}
+		else  {
         	error ("Invalid indirect block address (too large): %lu",
 			  (ULONG)addr);
+		}
     }    
 
     /*
@@ -702,11 +719,15 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], OFF_T length,
 		int prevcnt = todo_count;
 
 		if (getu32(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
-			if (flags & FS_FLAG_NOABORT)
+			if (flags & FS_FLAG_NOABORT) {
+				fprintf (stderr, "Invalid address in indirect list (too large): %lu",
+                  (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]));
 				return 0;
-			else
+			}
+			else {
 				error ("Invalid address in indirect list (too large): %lu",
                   (ULONG)getu32(fs, (u_int8_t *)&iaddr[n]));
+			}
         } 
 
         if (level == 1)
@@ -893,8 +914,8 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
 			  
 		fprintf(hFile, "%lu - %lu\n",
 		  (ULONG)cgdmin_lcl(fs, sb, i),
-		  (ULONG)(cgbase_lcl(fs, sb, i+1) - 1));
-
+          (((ULONG)cgbase_lcl(fs, sb, i + 1) - 1) < fs->last_block) ?
+		  ((ULONG)cgbase_lcl(fs, sb, i + 1) - 1) : fs->last_block);
 	}
 
 	return;
