@@ -137,7 +137,7 @@ extern int optind;
  /*
   * OpenBSD looks like BSD/OS 3.x.
   */
-#if defined(OPENBSD2)
+#if defined(OPENBSD2) || defined (OPENBSD3)
 #define SUPPORTED
 #define LSEEK		lseek
 #define OFF_T		off_t
