@@ -879,6 +879,9 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 					break;
 				}
 			}
+			if (!ntfs->mft_data) {
+				error ("$Data not found while loading the MFT");
+			}
 		}
 
 		/* Update the inode count based on the current size 
