@@ -501,7 +501,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		error ("ERROR: Directory index is sorted by type: %d.\nOnly $FNAME is currently supported", getu32(fs, idxroot->type));
 
 	/* Process it */
-	ntfs_dent_idxentry (ntfs, idxe, fs_data_root->buflen,
+	ntfs_dent_idxentry (ntfs, idxe, getu32(fs, idxroot->alloc),
 	  getu32(fs, idxroot->strlen), flags, action, ptr);
 
 
@@ -781,8 +781,8 @@ ntfs_find_file_rec (FS_INFO *fs, FS_DENT *fs_dent, FS_NAME *fs_name,
  */
 
 void
-ntfs_find_file (FS_INFO *fs, INUM_T inode_toid, int flags,
-  FS_DENT_WALK_FN action, char *ptr)
+ntfs_find_file (FS_INFO *fs, INUM_T inode_toid, u_int32_t type_toid, 
+  u_int32_t id_toid, int flags, FS_DENT_WALK_FN action, char *ptr)
 {
 
 	FS_INODE *fs_inode;
@@ -790,6 +790,7 @@ ntfs_find_file (FS_INFO *fs, INUM_T inode_toid, int flags,
 	FS_DENT *fs_dent = fs_dent_alloc(NTFS_MAXNAMLEN);
 	int myflags;
 	NTFS_INFO *ntfs = (NTFS_INFO *)fs;
+	char *attr = NULL;
 
 	/* sanity check */
     if (inode_toid < fs->first_inum || inode_toid > fs->last_inum)
@@ -814,11 +815,32 @@ ntfs_find_file (FS_INFO *fs, INUM_T inode_toid, int flags,
 	myflags = ((getu16(fs, ntfs->mft->flags) & NTFS_MFT_INUSE) ?
 		  FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
 
+	/* Get the name for the attribute - if specified */
+	if (type_toid != 0) {
+    	FS_DATA *fs_data = fs_data_lookup(fs_inode->attr, type_toid, id_toid);
+		if (!fs_data) {
+			error ("Type %d Id %d not found in MFT %d", type_toid, id_toid,
+			  inode_toid);
+		}
+
+		/* only add the attribute name if it is the non-default data stream */
+		if (strcmp (fs_data->name, "$Data") != 0)
+			attr = fs_data->name;
+	}
+
+
 	/* loop through all the names it may have */
 	for (fs_name = fs_inode->name; fs_name != NULL; fs_name = fs_name->next) {
-		
-		fs_dent->namlen = strlen (fs_name->name);
-		strncpy(fs_dent->name, fs_name->name, fs_dent->maxnamlen);
+
+		/* Append on the attribute name, if it exists */
+		if (attr != NULL) {
+			snprintf(fs_dent->name, fs_dent->maxnamlen, "%s:%s", 
+			  fs_name->name, attr);
+		}
+		else {
+			strncpy(fs_dent->name, fs_name->name, fs_dent->maxnamlen);
+		}
+		fs_dent->namlen = strlen (fs_dent->name);
 
 		/* if this is in the root directory, then call back */
 		if (fs_name->par_inode == NTFS_ROOTINO) {
