@@ -82,7 +82,7 @@ extern int optind;
  /*
   * OpenBSD looks like BSD/OS 3.x.
   */
-#if defined(OPENBSD2)
+#if defined(OPENBSD2) || defined(OPENBSD3)
 #define SUPPORTED
 #define STRTOUL		strtoul
 #endif
