@@ -36,7 +36,7 @@
 
 extern char *progname;
 
-FILE *logfp = stderr;
+FILE *logfp;
 
 void
 usage()
@@ -93,6 +93,7 @@ main(int argc, char **argv)
 	int 	ch;
 	daddr_t	sect_offset = 0;
 	u_int8_t flags = 0;
+	logfp = stderr;
 
 	progname = argv[0];
 
