@@ -22,28 +22,27 @@ typedef struct {
 		u_int8_t flag[2];
 	} part_meta[8];
 
-	u_int8_t bootinfo[3][4];
+	u_int8_t bootinfo[4][3];
+	u_int8_t reserved0[2];
 	u_int8_t sanity[4];
-	u_int8_t reserved0[40];
+	u_int8_t reserved1[38];
 	u_int8_t timestamp[8][4];
 	/* End VTOC */
 
 
 	u_int8_t write_reinstruct[2];
 	u_int8_t read_reinstruct[2];
-	u_int8_t reserved1[154];  
+	u_int8_t reserved2[154];  
 	u_int8_t rpm[2];
 	u_int8_t num_ph_cyl[2];
 	u_int8_t alt_per_cyl[2];
-	u_int8_t reserved2[2];
-	u_int8_t reserved3[2];
+	u_int8_t reserved3[4];
 	u_int8_t interleave[2];
 	u_int8_t num_cyl[2];
 	u_int8_t num_alt_cyl[2];
 	u_int8_t num_head[2];
 	u_int8_t sec_per_tr[2];
-	u_int8_t reserved4[2];
-	u_int8_t reserved5[2];
+	u_int8_t reserved5[4];
 
 	struct {
 		u_int8_t start_cyl[4];
@@ -91,13 +90,14 @@ typedef struct {
 	u_int8_t write_reinstruct[2];
 	u_int8_t read_reinstruct[2];
 	u_int8_t reserved1[8];
-	u_int8_t resereved2[12]; 
+	u_int8_t reserved2[12]; 
 	u_int8_t magic[2];
 	u_int8_t checksum[2];
 
 } sun_dlabel_i386;
 
 #define SUN_MAGIC	0xDABE
+#define SUN_SANITY	0x600DDEEE
 
 #define SUN_FLAG_UNMNT	0x01
 #define SUN_FLAG_RO		0x10
